/*
 * WItalk2 - italk client for Win32
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// DialogSounds.cxx


#include	<winsock2.h>
#include "DialogOrderedList.h"
#include "Sound.h"


class DialogSounds : public DialogOrderedList
{
  Sound editing_sound; // ҏW̉

  int select_sound(Sound &sound)
  {
    return DialogBoxParam(lgWindows::hInst(),
			  MAKEINTRESOURCE(rDialogSelectSound),
			  Hwnd(),
			  (DLGPROC)DialogSelectSoundDlgProc,
			  (LPARAM)&sound);
  }
  
public:
  DialogSounds(HWND _hwnd)
    : DialogOrderedList(_hwnd)
  {
  }
  
	// WM_INITDIALOG
	BOOL wmInitDialog(HWND focus, LPARAM lParam)
	{
		if( lParam ){
			PROPSHEETPAGE* pProp = (PROPSHEETPAGE*)lParam;
			m_pProfileManager = (ProfileManager*)pProp->lParam;
		}
		_ASSERT( m_pProfileManager != NULL );
		int value;
		m_pProfileManager->Read("PrivatePingInterval", value, 5 * 60);
		SetWindowText(Hwnd(rEditPrivatePingInterval), value);

		command = -1;
		wmCommand_ButtonSelectPing();

		registry_name      = "SoundPattern%d";
		new_list_string    = "*";
		hwnd_edit          = Hwnd(rEditSoundPattern);
		hwnd_button_new    = Hwnd(rButtonSoundPatternNew);
		hwnd_button_delete = Hwnd(rButtonSoundPatternDelete);
		hwnd_button_up     = Hwnd(rButtonSoundPatternUp);
		hwnd_button_down   = Hwnd(rButtonSoundPatternDown);
		hwnd_list          = Hwnd(rListSoundPattern);

		return DialogOrderedList::wmInitDialog(focus, lParam);
	}
  
  // OK ͍XV
  BOOL wmNotify_psnApply(void)
  {
    BOOL r = DialogOrderedList::wmNotify_psnApply();
    
    char buf[BUFSIZE];
    GetWindowText(rEditPrivatePingInterval, buf, sizeof(buf));
    m_pProfileManager->Write("PrivatePingInterval", atoi(buf));
    
    SendMessage(HwndPropertySheetOwner(), WM_DIALOGNOTIFY,
		rDialogSounds, TRUE);
    return r;
  }
  
  // WM_COMMAND
  BOOL wmCommand(int notify_code, int id, HWND hwnd_control)
  {
    command = notify_code;
    switch (id)
    {
#define c(x) case r##x: return wmCommand_##x()
      c(ButtonChooseSound);
      c(ButtonSelectPing);
      c(EditPrivatePingInterval);
#undef c
    }
    return DialogOrderedList::wmCommand(notify_code, id, hwnd_control);
  }
  
  void GetEditText(char *buf, int sizeof_buf)
  {
    GetWindowText(rEditSoundPattern, buf, sizeof_buf);
    char *c = buf + strlen(buf);
    *c++ = '\t';
    editing_sound.Encode(c, sizeof_buf - (c - buf));
  }
  
  void SetEditText(char *buf)
  {
    char *c = strchr(buf, '\t');
    if (c)
      *c++ = '\0';
    SetWindowText(rEditSoundPattern, buf);
    editing_sound.Decode(c);
  }
  
  BOOL wmCommand_ButtonChooseSound(void)
  {
    if (command == BN_CLICKED)
      if (select_sound(editing_sound))
      {
	command = EN_UPDATE;
	wmCommand_Edit();
      }
    return TRUE;
  }
  
  BOOL wmCommand_ButtonSelectPing(void)
  {
    if (command == BN_CLICKED || command == -1)
    {
      char buf[BUFSIZE];
      Sound ping;
      if (m_pProfileManager->Read("PingSound", buf, sizeof(buf)))
	ping.Decode(buf);
      else
      {
	int sound;
	m_pProfileManager->Read("PingBeep", sound, MB_OK);
	m_pProfileManager->Read("PingBeepFile", buf, sizeof(buf));
	ping.Set(sound, buf);
      }
      if (command == -1 || select_sound(ping))
      {
	ping.Encode(buf, sizeof(buf));
	m_pProfileManager->Write("PingSound", buf);
	SetWindowText(rEditSelectPing, buf);
	if (command != -1)
	  PropertySheetChanged();
      }
    }
    return TRUE; 
  }
  
  BOOL wmCommand_EditPrivatePingInterval(void)
  {
    if (command == EN_UPDATE)
      PropertySheetChanged();
    return TRUE;
  }
};


BOOL CALLBACK DialogSoundsDlgProc(HWND hwnd, UINT message,
				 WPARAM wParam, LPARAM lParam)
{
#define lgWindowClass DialogSounds
  if (!lgwc)
    switch (message)
    {
      lgwmInitDialog;
    }
  else
    switch (message)
    {
      lgwmCommand;
    }
  return lgwc->Default(message, wParam, lParam, TRUE);
}
