/*
 * RegEx - regular expression library (regex.c/.h) wrapper
 * Copyright (c) 2002 by Project Waffle / TSUJIKAWA Tohru <tsujikawa@tsg.ne.jp>
 *  
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// RegEx.h


#pragma	once


#include	"gnu/regex.h"

#include	<assert.h>
#include	<string.h>

#include	<vector>
#include	<string>
using namespace std;

#ifndef	_ASSERT
#define	_ASSERT(expr)	assert(expr)
#endif


class RegEx {

	bool		initialized;	// tO
	regex_t		reg;			// RpCϐK\p^[
	int			cflags;			// rtO
	regmatch_t	amatch[16];		// }b`
	const char*	lastmatch;		// Ō match Ƃ̃AhX
	char*		lastalloc;		// Kv widechar 񃏁[N

public:

	//
	// constructor
	//
	//   K\RpC
	//
	RegEx(const char* pattern, int cflags = 0) {
		_ASSERT(pattern && "pattern string is NULL.");
		initialized = regcomp(&reg, pattern, cflags) == 0;
		_ASSERT(initialized && "pattern string may be malformed.");
		lastmatch = lastalloc = NULL;
	}
	RegEx(const string& pattern, int cflags = 0) { RegEx(pattern.c_str(), cflags); }
	RegEx(const wchar_t* pattern, int cflags = 0) {
		_ASSERT(pattern && "pattern string is NULL.");
		size_t	mbslen = wcstombs(NULL, pattern, 0);
		_ASSERT(mbslen >= 0 && "pattern string contains invalid unicode characters.");
		if (mbslen < 0) { initialized = false; return; }
		char*	mbspattern = (char*)_alloca(mbslen+1);
		wcstombs(mbspattern, pattern, wcslen(pattern)+1);
		RegEx(mbspattern, cflags);
	}
	RegEx(const wstring& pattern, int cflags = 0) {
		size_t	mbslen = wcstombs(NULL, pattern.c_str(), 0);
		_ASSERT(mbslen >= 0 && "pattern string contains invalid unicode characters.");
		if (mbslen < 0) { initialized = false; return; }
		char*	mbspattern = (char*)_alloca(mbslen+1);
		wcstombs(mbspattern, pattern.c_str(), pattern.size()+1);
		RegEx(mbspattern, cflags);
	}

	//
	// destructor
	//
	//   K\̓[N
	//
	~RegEx() {
		if (initialized) regfree(&reg);
		/*if (lastalloc)*/ delete [] lastalloc;
	}

	//
	// match
	//
	//   RpCς̐K\p^[ƕrC}b`邩ԂB
	//
	static bool match(const char* str, const char* pattern, int cflags = 0) {
		_ASSERT(str && "target string is NULL."); _ASSERT(pattern && "pattern string is NULL.");
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const char* str, const string& patten, int cflags = 0) { return match(str, patten.c_str(), cflags); }
	static bool match(const char* str, const wchar_t* pattern, int cflags = 0) {
		_ASSERT(str && "target string is NULL."); _ASSERT(pattern && "pattern string is NULL.");
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const char* str, const wstring& pattern, int cflags = 0) {
		_ASSERT(str && "target string is NULL.");
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const string& str, const char* pattern, int cflags = 0) { return match(str.c_str(), pattern, cflags); }
	static bool match(const string& str, const string& pattern, int cflags = 0) { return match(str.c_str(), pattern.c_str(), cflags); }
	static bool match(const string& str, const wchar_t* pattern, int cflags = 0) { return match(str.c_str(), pattern, cflags); }
	static bool match(const string& str, const wstring& pattern, int cflags = 0) {
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const wchar_t* str, const char* pattern, int cflags = 0) {
		_ASSERT(str && "target string is NULL."); _ASSERT(pattern && "pattern string is NULL.");
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const wchar_t* str, const string& pattern, int cflags = 0) { return match(str, pattern.c_str(), cflags); }
	static bool match(const wchar_t* str, const wchar_t* pattern, int cflags = 0) {
		_ASSERT(str && "target string is NULL."); _ASSERT(pattern && "pattern string is NULL.");
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const wchar_t* str, const wstring& pattern, int cflags = 0) {
		_ASSERT(str && "target string is NULL.");
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const wstring& str, const char* pattern, int cflags = 0) {
		_ASSERT(pattern && "pattern string is NULL.");
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const wstring& str, const string& pattern, int cflags = 0) {
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const wstring& str, const wchar_t* pattern, int cflags = 0) {
		_ASSERT(pattern && "pattern string is NULL.");
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	static bool match(const wstring& str, const wstring& pattern, int cflags = 0) {
		RegEx	re(pattern, cflags);
		return re.match(str);
	}
	bool match(const char* str) {
		_ASSERT(str && "target string is NULL.");
		lastmatch = str;
		return (initialized && (regexec(&reg, str, 16, &amatch[0], 0) == 0));
	}
	bool match(const string& str) { return match(str.c_str()); }
	bool match(const wchar_t* str) {
		/* if (lastalloc)*/ delete [] lastalloc;
		_ASSERT(str && "target string is NULL.");
		size_t	mbslen = wcstombs(NULL, str, 0);
		_ASSERT(mbslen >= 0 && "target string contains invalid unicode characters.");
		if (mbslen < 0) return false;
		lastalloc = new char[mbslen+1];
		wcstombs(lastalloc, str, wcslen(str)+1);
		return match(lastalloc);
	}
	bool match(const wstring& str) {
		/* if (lastalloc)*/ delete [] lastalloc;
		size_t	mbslen = wcstombs(NULL, str.c_str(), 0);
		_ASSERT(mbslen >= 0 && "target string contains invalid unicode characters.");
		if (mbslen < 0) return false;
		lastalloc = new char[mbslen+1];
		wcstombs(lastalloc, str.c_str(), str.size()+1);
		return match(lastalloc);
	}

	//
	// match ӏ̏擾郁o֐Q
	//

	//
	// ChJ[h̔ԍw肵ĊJnʒu擾
	//
	//   wchar n֐ match ꍇ́CRȂ wchar ł͐Ȃ index ɂȂB
	//
	int spos(int nth) { _ASSERT(nth < 16 && "nth must be below 16"); return (int)amatch[nth].rm_so; }

	//
	// Iʒu擾
	//
	//   wchar n֐ match ꍇ́CRȂ wchar ł͐Ȃ index ɂȂB
	//
	int epos(int nth) { _ASSERT(nth < 16 && "nth must be below 16"); return (int)amatch[nth].rm_eo; }

	//
	// match 𕡎
	//
	//   ʃoCgԂB
	//
	//   match ֐͌Ăяoꂽƕ̃|C^LĂC̃|C^ɑ΂đsB
	//
	//   wchar n֐̏ꍇ͓ɕϊ̕ێĂ̂ŁCIuWFNgɂ킽Ďgpł邪C
	//   char n֐̏ꍇ͒Pɓnꂽւ̃|C^ێĂɂȂ̂ŁCmatch Ƃ
	//   񂪗LȊԂgpłȂB
	//
	int get(char* buf, int sizeofbuf, int nth) {
		_ASSERT(spos(nth) != -1 && "wildcard doesn't matched.");
		int		cpylen = epos(nth) - spos(nth) + 1;
		if (cpylen > sizeofbuf-1) cpylen = sizeofbuf-1;
		memcpy(buf, lastmatch+spos(nth), cpylen); buf[cpylen] = 0;
		return cpylen;
	}

	string get(int nth) {
		_ASSERT(spos(nth) != -1 && "wildcard doesn't matched.");
		string	s(lastmatch, spos(nth), epos(nth)-spos(nth)+1);
		return s;
	}

	vector<string> get() {
		vector<string>	r;
		for (int i = 0; spos(i) >= 0 && i < 16; i++) r.push_back(get(i));
		return r;
	}

};
