// Italk Protocol 1.0 and biff-port support modifications.
// Copyright (C) 1998, SAKAMOTO Takahiro <takas@is.s.u-tokyo.ac.jp>

/*
 * WItalk2 - italk client for Win32
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// WItalk2URL.cxx

#include	<winsock2.h>
#include <crtdbg.h>

#include "lgFiles.h"
#include "WItalk2.h"
#include "DialogURL.h"
#include "ProfileManager.h"

// 
void WItalk2::url_create(void)
{
	_ASSERT( m_pProfileManager );
  BOOL does_show_open_url;
  m_pProfileManager->Read("DoesShowOpenURL", does_show_open_url, FALSE);

  TOOLINFO ti;
  ti.cbSize   = sizeof(TOOLINFO);
  ti.uFlags   = TTF_IDISHWND | TTF_SUBCLASS;
  ti.hinst    = NULL;

  // 
  hwnd_url = CreateWindowEx(NULL, TOOLBARCLASSNAME /* "STATIC" */, NULL,
							WS_CHILD | /* SS_SIMPLE | SS_NOPREFIX |*/ (does_show_open_url ? WS_VISIBLE : 0),
							0, 0, 1, 1, Hwnd(), NULL, lgWindows::hInst(), NULL);
  _ASSERT(hwnd_url);

  // TuNX
  subclass_wmCtlColor(hwnd_url);

  // R{{bNX
  hwnd_combo_url = CreateWindowEx(NULL, "COMBOBOX", NULL,
								  WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_TABSTOP | CBS_DROPDOWN | CBS_AUTOHSCROLL,
								  0, 0, 1, 1, hwnd_url, NULL, lgWindows::hInst(), NULL);
  _ASSERT(hwnd_combo_url);

  ti.hwnd     = hwnd_combo_url;
  ti.uId      = (UINT)ti.hwnd;
  ti.lpszText = LoadString(rStringURLList);
  SendMessage(hwnd_tooltip, TTM_ADDTOOL, 0, (LPARAM)&ti);

  // {^
  hwnd_button_url = CreateWindowEx(NULL, "BUTTON", NULL,
								   WS_CHILD | WS_VISIBLE | BS_BITMAP,
								   0, 0, 1, 1, hwnd_url, NULL, lgWindows::hInst(), NULL);
  _ASSERT(hwnd_button_url);
  SendMessage(hwnd_button_url, BM_SETIMAGE,
	      IMAGE_BITMAP, (LPARAM)hbitmap_go);

  ti.hwnd     = hwnd_button_url;
  ti.uId      = (UINT)ti.hwnd;
  ti.lpszText = LoadString(rStringOpenURL);
  SendMessage(hwnd_tooltip, TTM_ADDTOOL, 0, (LPARAM)&ti);

  // ACR
  hwnd_icon_url = CreateWindowEx(NULL, "STATIC", NULL,
								 WS_CHILD | WS_VISIBLE | SS_ICON,
								 0, 0, 1, 1, hwnd_url, NULL, lgWindows::hInst(), NULL);
  _ASSERT(hwnd_icon_url);

  hicon_url = lgRegistry::GetAssociatedIcon(".html");
  if (hicon_url)
    Static_SetIcon(hwnd_icon_url, hicon_url);
  else
    ShowWindow(hwnd_icon_url, SW_HIDE);

  hfont_combo_url = NULL;

  url_set_font();
}


// Ƃ܂
void WItalk2::url_destroy(void)
{
  hwnd_url = NULL;
  hwnd_combo_url = NULL;
  hwnd_icon_url = NULL;

  if (hfont_combo_url)
    DeleteObject(hfont_combo_url);
  hfont_combo_url = NULL;

  hicon_url = NULL; // TODO: ACR͔pȂĂ̂낤H
}


// TCY combo_connectto_resize ɌĂԂ
void WItalk2::url_resize(void)
{
  RECT rc;
  GetClientRect(Hwnd(), &rc);
  SendMessage(hwnd_url, WM_SIZE, 0, MAKELONG(rc.right, rc.bottom));
  GetChildWindowRect(toolbar->Hwnd(), &rc);
  MoveWindow(hwnd_url, rc.left, rc.bottom, FALSE);

  GetChildWindowRect(hwnd_combo_connectto, &rc);
  rc.bottom += 256;
  MoveWindow(hwnd_combo_url, &rc, FALSE);

  GetChildWindowRect(hwnd_button_connectto, &rc);
  MoveWindow(hwnd_button_url, &rc, FALSE);

  GetChildWindowRect(hwnd_anim_connectto, &rc);
  MoveWindow(hwnd_icon_url, &rc, FALSE);

  url_redraw();
}


// ĕ`
void WItalk2::url_redraw(void)
{
  InvalidateRect(hwnd_url, NULL, TRUE);
  UpdateWindow(hwnd_url);
}


// tHg̐ݒ
void WItalk2::url_set_font(void)
{
	_ASSERT( m_pProfileManager );
  LOGFONT lf;
  m_pProfileManager->Read("Font", lf, lgWindows::LoadString(rStringDefaultLogFont));
  if (hfont_combo_url)
    DeleteObject(hfont_combo_url);
  hfont_combo_url = CreateFontIndirect(&lf);

  SetWindowFont(hwnd_combo_url, hfont_combo_url, TRUE);

  url_resize();
}


// tH[JX̐ݒ
void WItalk2::url_setfocus(void)
{
  ShowWindow(hwnd_url, SW_SHOW);
  splitbar_resize();
  SetFocus(hwnd_combo_url);
}


// tH[JXĂ邩ǂ
BOOL WItalk2::url_is_focused(void)
{
  HWND hwnd_focus = GetFocus();
  while (hwnd_focus != hwnd_combo_url)
    if (!(hwnd_focus = GetParent(hwnd_focus)))
      return FALSE;
  return TRUE;
}


// URL J
BOOL WItalk2::url_open(BOOL force_open)
{
	if (!force_open && !url_is_focused()) return FALSE;

	char url[BUFSIZE];
	GetWindowText(hwnd_combo_url, url, sizeof(url));
	lgFiles::OpenURL(url, DialogURL::url_execmode);

	char buf[BUFSIZE];
	for (int i = 0; ; i++) if (CB_ERR == ComboBox_GetLBText(hwnd_combo_url, i, buf)) {
		ComboBox_InsertString(hwnd_combo_url, 0, url);
		break;
	} else if (_mbsequal(url, buf)) {
		break;
	}
	reset_focus();

	return TRUE;
}


// URL ɒǉ
void WItalk2::url_add(const wchar_t *url)
{
  if (!url)
  {
    url_setfocus();
    return;
  }

  int len;
  char *buf;
  alloca_wcstombs(buf, url, len);
  ComboBox_InsertString(hwnd_combo_url, 0, buf);
  SetWindowText(hwnd_combo_url, buf);

/*
  BOOL is_checked;
  registry.Read("AutoOpenURL", is_checked, FALSE);

  if (is_checked)
  {
    url_open(TRUE);
    ShowWindow(hwnd_url, SW_SHOW);
    splitbar_resize();
  }
*/
}


// \ / \
void WItalk2::url_toggle_display(void)
{
	_ASSERT( m_pProfileManager );
  if (url_is_displaied())
    ShowWindow(hwnd_url, SW_HIDE);
  else
    ShowWindow(hwnd_url, SW_SHOW);
  splitbar_resize();
  
  m_pProfileManager->Write("DoesShowOpenURL", url_is_displaied());
}


BOOL WItalk2::url_is_displaied(void)
{
  return IsWindowVisible(hwnd_url);
}
