/*
 * LIBGANA - library for GANA
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// lgString.cxx


#include <stdlib.h>
//#include <varargs.h>
#include "lgWindows.h"
#include "gnu/regex.h"


// POSIX K\Cugp match
//
//   GNU glibc2.2.5 R
//
bool lgString::match(const char* mbs, const char* pattern, int cflags)
{
	regex_t	preg;
	int ar = regcomp(&preg, pattern, cflags);
	_ASSERT(!ar);
	bool	r = regexec(&preg, mbs, 0, NULL, cflags) == 0;
	regfree(&preg);

	return r;
}

bool lgString::match(const wchar_t* wcs, const wchar_t* pattern, int cflags)
{
	size_t	plen;
	char*	pmbs = new char[plen = wcstombs(NULL, pattern, NULL)+1];

	wcstombs(pmbs, pattern, plen);

	bool r = match(wcs, pmbs, cflags);
	
	delete [] pmbs;
	return r;
}

bool lgString::match(const wchar_t* wcs, const char* pmbs, int cflags)
{
	size_t	tlen;
	char*	tmbs = new char[tlen = wcstombs(NULL, wcs, NULL)+1];

	wcstombs(tmbs, wcs, tlen);

	bool r = match(tmbs, pmbs, cflags);
	
	delete [] tmbs;
	return r;
}


string rprintf(const char* format, ...)
{
	va_list		params;
	va_start(params, format);

	char	buf[1024];
	vsprintf(buf, format, params);

	va_end(params);
	return string(buf);
}


// UNIX ṼChJ[h}b`O
//   mbs1: mbs2 ̏WɊ܂܂邩ǂ𒲂ׂΏ
//   mbs2: W
//   case_sensitive: 啶ʂ邩ǂ
// p:
//   BOOL f = lgString::DoesMatch("hoehoe nexe", "ho?*hoe*");
//   ̏ꍇ f  TRUE
// Ԓl:
//   return: mbs2 ̏W mbs1 ܂܂Ȃ (܂}b`) TRUE
//   i_end_r: ̓}b`ꏊ̐擪̕
//   c_end_r: ̓}b`ꏊ̍Ōւ̃|C^
// l:
//   mbs1  mbs2 ɑOv̂Ƃ TRUE
BOOL lgString::DoesMatch(const char *mbs1, const char *mbs2,
				   int &i_end_r, char *&c_end_r,
				   BOOL case_sensitive)
{
  if (!mbs1 || !mbs2)
    return FALSE;
  int i_end = 0;
  while (1)
  {
    if (!*mbs1)
    {
      if (!*mbs2 || (*mbs2 == '*' && !*(mbs2 + 1)))
      {
	i_end_r = i_end;
	c_end_r = (char *)mbs1;
	return TRUE;
      }
      else
	return FALSE;
    }
    if (!*mbs2)
    {
      i_end_r = i_end;
      c_end_r = (char *)mbs1;
      return TRUE;
    }
    else if (*mbs2 == '*')
    {
      mbs2++;
      if (!*mbs2)
      {
	i_end_r = i_end;
	c_end_r = (char *)mbs1;
	return TRUE;
      }
      while (1)
      {
	int _i_end;
        if (lgString::DoesMatch(mbs1, mbs2, _i_end, c_end_r, case_sensitive))
	{
	  i_end_r = i_end + _i_end;
	  return TRUE;
	}
        mbs1 = _mbsinc(mbs1);
	i_end++;
        if (!*mbs1)
	  return FALSE;
      }
    }
    else if ((case_sensitive && _mbcequal(mbs1, mbs2)) ||
	     (!case_sensitive && _mbciequal(mbs1, mbs2)) ||
	     *mbs2 == '?')
    {
      mbs1 = _mbsinc(mbs1);
      i_end++;
      mbs2 = _mbsinc(mbs2);
      continue;
    }
    return FALSE;
  }
}


BOOL lgString::DoesMatch(const wchar_t *wcs1, const wchar_t *wcs2,
				   int &i_end_r, wchar_t *&c_end_r,
				   BOOL case_sensitive)
{
	if (!wcs1 || !wcs2) return FALSE;

	int i_end = 0;

	while (1){
		if (!*wcs1){
			if (!*wcs2 || (*wcs2 == L'*' && !*(wcs2 + 1))){
				i_end_r = i_end;
				c_end_r = (wchar_t *)wcs1;
				return TRUE;
			} else return FALSE;
		}
		if (!*wcs2){
			i_end_r = i_end;
			c_end_r = (wchar_t *)wcs1;
			return TRUE;
		} else if (*wcs2 == L'*') {
			wcs2++;
			if (!*wcs2){
				i_end_r = i_end;
				c_end_r = (wchar_t *)wcs1;
				return TRUE;
			}
			while (1){
				int _i_end;
				if (lgString::DoesMatch(wcs1, wcs2, _i_end, c_end_r, case_sensitive)){
					i_end_r = i_end + _i_end;
					return TRUE;
				}
				wcs1++; i_end++;
				if (!*wcs1) return FALSE;
			}
		} else if ((case_sensitive && *wcs1 == *wcs2) || (!case_sensitive && towlower(*wcs1) == towlower(*wcs2)) || *wcs2 == L'?'){
			wcs1++; i_end++; wcs2++;
			continue;
		}
		return FALSE;
	}
}

// GANA C˗BR[h󂯂ƂĒuB(Waffle 0.1.6)
//
// euc ɕϊłȂ "" ɕϊB

#if 0

void lgString::sjisc2eucc(BYTE *euc_jp, const BYTE *sjis)
{
  BYTE b1 = sjis[0];
  BYTE b2 = sjis[1];
  if (0xe0 <= b1)
    b1 -= 0x40;
  b1 = (BYTE)(((int)b1 - 0x81) * 2 + 0x21);
  if (0x80 <= b2)
    b2--;
  b2 -= 0x1f;
  if (0x7f <= b2)
    b2 -= 0x5e, b1++;
  euc_jp[0] = (BYTE)(b1 | 0x80);
  euc_jp[1] = (BYTE)(b2 | 0x80);
}

#else

void lgString::sjisc2eucc(BYTE *euc_jp, const BYTE *sjis)
{
    BYTE b1 = sjis[0];
    BYTE b2 = sjis[1];
    if (0xe0 <= b1)
      b1 -= 0x40;
    b1 = (BYTE)(((int)b1 - 0x81) * 2 + 0x21);
    if (0x80 <= b2)
      b2--;
    b2 -= 0x1f;
    if (0x7f <= b2)
      b2 -= 0x5e, b1++;
    if (0x20 + 94 < b1) // 94 ȏ͋֎~()
    {
      euc_jp[0] = 0xa2;
      euc_jp[1] = 0xae;
    }
    else
    {
      euc_jp[0] = (BYTE)(b1 | 0x80);
      euc_jp[1] = (BYTE)(b2 | 0x80);
    }
}

#endif

// euc_jp ̃oCgԂ ('\0' ͊܂܂)
int lgString::sjis2euc(BYTE *euc_jp, const BYTE *sjis,
				 BOOL does_convert_crlf2lf)
{
	int sizeof_euc_jp = 0;
	while (*sjis) {
		if (0xa0 <= *sjis && *sjis < 0xe0) { // J^Ji
			if (euc_jp) {
				*(euc_jp++) = 0x8e;
				*(euc_jp++) = *(sjis++);
			} else sjis++;
			sizeof_euc_jp += 2;
		} else if (*sjis & 0x80) {// 
			if (!sjis[1]) break;// 2 oCgڂȂ
			if (euc_jp) {
				sjisc2eucc(euc_jp, sjis);
				euc_jp += 2;
			}
			sizeof_euc_jp += 2;
			sjis += 2;
		} else if (does_convert_crlf2lf && *sjis == '\r' && *(sjis + 1) == '\n') {
			if (euc_jp) *(euc_jp++) = '\n';
			sizeof_euc_jp ++;
			sjis += 2;
		} else {
			if (euc_jp) *(euc_jp++) = *sjis;
			sizeof_euc_jp++;
			sjis++;
		}
	}
	if (euc_jp) *euc_jp = '\0';
	return sizeof_euc_jp;
}


// euc_jp ̃oCgԂ ('\0' ͊܂܂)
int lgString::uni2euc(BYTE *euc_jp, const wchar_t *uni,
				BOOL does_convert_crlf2lf)
{
  int sizeof_euc_jp = 0;
  BYTE buf[2];
  int len;
  while (*uni && 0 < (len = wctomb((char *)buf, *uni)))
    if (len == 1 && 0xa0 <= *buf && *buf < 0xe0) // J^Ji
    {
      if (euc_jp)
      {
	*(euc_jp++) = 0x8e;
	*(euc_jp++) = *buf;
      }
      sizeof_euc_jp += 2;
      uni++;
    }
    else if (len == 2) // 
    {
      if (euc_jp)
      {
	sjisc2eucc(euc_jp, buf);
	euc_jp += 2;
      }
      sizeof_euc_jp += 2;
      uni ++;
    }
    else if (does_convert_crlf2lf && uni[0] == L'\r' && uni[1] == L'\n')
    {
      if (euc_jp)
	*(euc_jp++) = '\n';
      sizeof_euc_jp++;
      uni += 2;
    }
    else
    {
      if (euc_jp)
	*(euc_jp++) = *buf;
      sizeof_euc_jp++;
      uni++;
    }
  if (euc_jp)
    *euc_jp = '\0';
  return sizeof_euc_jp;
}


void lgString::eucc2sjisc(BYTE *sjis, const BYTE *euc_jp)
{
  WORD b1 = (WORD)(euc_jp[0] & 0x7f);
  WORD b2 = (WORD)(euc_jp[1] & 0x7f);
  if (b1 % 2 == 0)
    b1--, b2 += 0x5e;
  b1 = (WORD)((b1 - 0x21) / 2 + 0x81);
  b2 += 0x1f;
  if (0x7f <= b2)
    b2++;
  if (0xa0 <= b1)
    b1 += 0x40;
  sjis[0] = (BYTE)b1;
  sjis[1] = (BYTE)b2;
}


// sjis ̃oCgԂ ('\0' ͊܂܂)
int lgString::euc2sjis(BYTE *sjis, const BYTE *euc_jp,
				 BOOL does_convert_lf2crlf)
{
  int sizeof_sjis = 0;
  while (*euc_jp)
    if (*euc_jp == 0x8e) // J^Ji
    {
      euc_jp++;
      if (!*euc_jp)
	break;
      if (sjis)
	*(sjis++) = *euc_jp;
      sizeof_sjis++;
      euc_jp++;
    }
    else if (*euc_jp == 0x8f) // ⏕
    {
      if (!euc_jp[1] || !euc_jp[2])
	break;
      euc_jp += 3;
    }
    else if (*euc_jp & 0x80) // 
    {
      if (!euc_jp[1])
	break;
      if (sjis)
      {
	eucc2sjisc(sjis, euc_jp);
	sjis += 2;
      }
      sizeof_sjis += 2;
      euc_jp += 2;
    }
    else if (does_convert_lf2crlf && *euc_jp == '\n')
    {
      if (sjis)
      {
	*(sjis++) = '\r';
	*(sjis++) = '\n';
      }
      sizeof_sjis += 2;
      euc_jp++;
    }
    else
    {
      if (sjis)
	*(sjis++) = *euc_jp;
      sizeof_sjis ++;
      euc_jp++;
    }
  if (sjis)
    *sjis = '\0';
  return sizeof_sjis;
}


// uni ̕Ԃ (L'\0' ͊܂܂)
int lgString::euc2uni(wchar_t *uni, const BYTE *euc_jp,
				BOOL does_convert_lf2crlf)
{
  BYTE buf[2];
  int lengthof_uni = 0;
  while (*euc_jp)
    if (*euc_jp == 0x8e) // J^Ji
    {
      euc_jp++;
      if (!*euc_jp)
	break;
      if (uni)
      {
	mbtowc(uni, (const char *)euc_jp, 1);
	uni++;
      }
      lengthof_uni++;
      euc_jp++;
    }
    else if (*euc_jp == 0x8f) // ⏕
    {
      if (!euc_jp[1] || !euc_jp[2])
	break;
      euc_jp += 3;
    }
    else if (*euc_jp & 0x80) // 
    {
      if (!euc_jp[1])
	break;
      if (uni)
      {
	eucc2sjisc(buf, euc_jp);
	mbtowc(uni, (const char *)buf, 2);
	uni++;
      }
      lengthof_uni++;
      euc_jp += 2;
    }
    else if (does_convert_lf2crlf && *euc_jp == '\n')
    {
      if (uni)
      {
	*(uni++) = L'\r';
	*(uni++) = L'\n';
      }
      lengthof_uni += 2;
      euc_jp++;
    }
    else
    {
      if (uni)
      {
	mbtowc(uni, (const char *)euc_jp, 1);
	uni++;
      }
      lengthof_uni++;
      euc_jp++;
    }
  if (uni)
    *uni = L'\0';
  return lengthof_uni;
}


// SJIS ǂ
BOOL lgString::IsSJIS(const char *str)
{
  if (!str)
    return FALSE;
  while (*str)
  {
    if (_ismbblead((BYTE)*str))
    {
      str++;
      if (!_ismbbtrail((BYTE)*str))
	return FALSE;
    }
    str++;
  }
  return TRUE;
}


// str 񒆂 from ꂽ to ɒu
char *lgString::ReplaceChar(char *str, char to, char from)
{
  if (str)
  {
    char *c = str;
    while ((c = strchr(c, from)))
      *c++ = to;
  }
  return str;
}


// Xy[X (' ',  '\t', '\r', '\n') ΂
char *lgString::SkipSpace(const char *str)
{
  if (!str)
    return NULL;
  while (*str == ' ' || *str == '\t' || *str == '\r' || *str == '\n')
    str++;
  return (char *)str;
}

wchar_t *lgString::SkipSpace(const wchar_t *str)
{
  if (!str)
    return NULL;
  while (*str == L' ' || *str == L'\t' || *str == L'\r' || *str == L'\n')
    str++;
  return (wchar_t *)str;
}


//  ('0'`'9') ΂
char *lgString::SkipNumber(const char *str)
{
  if (!str)
    return NULL;
  while ('0' <= *str && *str <= '9')
    str++;
  return (char *)str;
}

wchar_t *lgString::SkipNumber(const wchar_t *str)
{
  if (!str)
    return NULL;
  while (L'0' <= *str && *str <= L'9')
    str++;
  return (wchar_t *)str;
}


// URL ɌƂ؂o
wchar_t *lgString::GetURL(wchar_t *url, int lengthof_url,
				 const wchar_t *str)
{
  wchar_t *match_end;
  wchar_t *start = NULL;
  if (FALSE) ;
#define PROTOCOL(P)						\
  else if (lgString::DoesMatch(str, L"*" P, match_end))		\
    start =  match_end - (sizeof(P) / 2 - 1)
  PROTOCOL(L"file://");
  PROTOCOL(L"ftp://");
//  PROTOCOL(L"gopher://");
  PROTOCOL(L"http://");
  PROTOCOL(L"ttp://");
  PROTOCOL(L"https://");
  PROTOCOL(L"mailto:");
//  PROTOCOL(L"news:");
//  PROTOCOL(L"rlogin://");
//  PROTOCOL(L"snews:");
  PROTOCOL(L"telnet://");
//  PROTOCOL(L"tn3270://");
  PROTOCOL(L"italk://");
  PROTOCOL(L"\\\\");
#undef PROTOCOL
  else
    return NULL;
  
  wchar_t *s = start;
  while (*s)
    if (*s <= L' ' || L'\x7f' <= *s || *s == L'"' || *s == L'>' || *s == L'<')
      break;
    else
      s++;
  if (lengthof_url <= s - start)
    return NULL;
  wcsncpy(url, start, s - start);
  url[s - start] = L'\0';
  return url;
}


char *lgString::GetURL(char *url, int sizeof_url,
				 const char *mbs)
{
  int sizeof_mbs = strlen(mbs) + 1;
  int size = mbstowcs(NULL, mbs, sizeof_mbs);
  if (size <= 0)
    return NULL;
  wchar_t *wcs  = (wchar_t *)_alloca((size + 1) * sizeof(wchar_t));
  sizeof_url /= 2;
  wchar_t *wurl = (wchar_t *)_alloca(sizeof_url * sizeof(wchar_t));
  mbstowcs(wcs, mbs, sizeof_mbs);
  wchar_t *r = GetURL(wurl, sizeof_url, wcs);
  if (!r)
    return NULL;
  wcstombs(url, wurl, sizeof_url);
  return url;
}


// _mbsstr ̑啶ʂȂo[W
BYTE *lgString::_mbsistr(const BYTE *mbs1, const BYTE *mbs2)
{
  if (mbs1 && mbs2)
    while (*mbs1)
    {
      if (_mbciequal((char *)mbs1, (char *)mbs2))
      {
	const BYTE *s1 = mbs1;
	const BYTE *s2 = mbs2;
	while (_mbciequal((char *)s1, (char *)s2) && *s2)
	{
	  s1 = _mbsinc(s1);
	  s2 = _mbsinc(s2);
	}
	if (!*s2)
	  return (BYTE *)mbs1;
      }
      mbs1 = _mbsinc(mbs1);
    }
  return NULL;
}
