/*
 * LIBGANA - library for GANA
 * Copyright (C) 1996-1997, TAGA Nayuta <nayuta@is.s.u-tokyo.ac.jp>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

///////////////////////////////////////////////////////////////////////////////
// lgcAnimation.cxx


#include <crtdbg.h>

#include "lgcAnimation.h"
#include "lgGUIParts.h"


LRESULT CALLBACK lgcAnimationWndProc(HWND hwnd, UINT message,
				     WPARAM wParam, LPARAM lParam);


class lgcAnimation : public lgWindow
{
  int timer_id;
  HIMAGELIST himl;
  int i;
  BOOL has_boarder;
  
  void resize(void)
  {
    if (himl)
    {
      int cx, cy;
      ImageList_GetIconSize(himl, &cx, &cy);
      if (has_boarder)
	cx += 4, cy += 4;
      RECT rc;
      GetChildWindowRect(Hwnd(), &rc);
      ::MoveWindow(Hwnd(), rc.left, rc.top, cx, cy, FALSE);
    }
  }
  
public:
  lgcAnimation(HWND _hwnd)
    : lgWindow(_hwnd), timer_id(0), himl(NULL), has_boarder(FALSE) {}
  
  ~lgcAnimation()
  {
    if (timer_id)
      KillTimer(Hwnd(), timer_id);
  }
  
  int animationMessage(WPARAM wParam, LPARAM lParam)
  {
    switch (wParam)
    {
      case lgcAnimation_WPARAM_SETIMAGELIST:
	himl = (HIMAGELIST)lParam;
	resize();
	break;
      case lgcAnimation_WPARAM_BORDER:
	has_boarder = (BOOL)lParam;
	resize();
	break;
      case lgcAnimation_WPARAM_START:
	timer_id = SetTimer(Hwnd(), 1, (int)lParam, NULL);
	break;
      case lgcAnimation_WPARAM_END:
	if (timer_id)
	  KillTimer(Hwnd(), timer_id);
	break;
    }
    i = 0;
    InvalidateRect(Hwnd(), NULL, FALSE);
    UpdateWindow(Hwnd());
    return 0;
  }
  
  int wmPaint(void)
  {
    PAINTSTRUCT ps;
    HDC hdc = BeginPaint(Hwnd(), &ps);
    if (himl)
      if (has_boarder)
      {
	ImageList_Draw(himl, i, hdc, 2, 2, ILD_NORMAL);
	RECT rc;
	GetClientRect(Hwnd(), &rc);
	lgGUIParts::Draw3DFrameInside(hdc, rc, lgGUIParts::FrameDown2);
      }
      else
	ImageList_Draw(himl, i, hdc, 0, 0, ILD_NORMAL);
    EndPaint(Hwnd(), &ps);
    return 0;
  }
  
  int wmTimer(int /* id */, TIMERPROC * /* timer_proc */)
  {
    i++;
    i %= ImageList_GetImageCount(himl);
    InvalidateRect(Hwnd(), NULL, FALSE);
    UpdateWindow(Hwnd());
    return 0;
  }
  
public:
  static ATOM Register(void)
  {
    WNDCLASSEX wc;
	wc.cbSize        = sizeof(WNDCLASSEX);
    wc.style         = 0;
    wc.lpfnWndProc   = lgcAnimationWndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = sizeof(lgcAnimation *);
    wc.hInstance     = lgWindows::hInst();
    wc.hIcon         = NULL;
	wc.hIconSm       = NULL;
    wc.hCursor       = NULL;
    wc.hbrBackground = NULL;
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = WC_lgcAnimation;
    ATOM rrcex = RegisterClassEx(&wc);
	_ASSERT(rrcex);
	return rrcex;
  }
};


LRESULT CALLBACK lgcAnimationWndProc(HWND hwnd, UINT message,
				     WPARAM wParam, LPARAM lParam)
{
#define lgWindowClass lgcAnimation
  if (lgwc)
    switch (message)
    {
      lgwmDestroy;
      lgwmPaint;
      lgwmTimer;
      case lgcAnimation_MESSAGE:
	return lgwc->animationMessage(wParam, lParam);
    }
  else
    switch (message)
    {
      lgwmCreate;
    }
  return DefWindowProc(hwnd, message, wParam, lParam);
}


// Animation NX̓o^
ATOM Register_lgcAnimation(void)
{
  return lgcAnimation::Register();
}
